@@ -23,17 +23,18 @@
 #include <stdlib.h>
 #include <errno.h>
 
-#ifndef ENOERR
-#define ENOERR 0
+#ifndef NC_NOERR
+#define NC_NOERR 0
 #endif
 
 #include <sys/types.h>
 #include <sys/stat.h>
+
 #include <fcntl.h>
 #include <string.h>
 
 #ifndef HAVE_SSIZE_T
-#define ssize_t int
+typedef int ssize_t;
 #endif
 
 #ifndef SEEK_SET
@@ -99,6 +100,41 @@ static int ncio_spx_close(ncio *nciop, int doUnlink);
 #ifndef POSIXIO_DEFAULT_PAGESIZE
 #define POSIXIO_DEFAULT_PAGESIZE 4096
 #endif
+
+/*! Cross-platform file length.
+ *
+ * Some versions of Visual Studio are throwing errno 132
+ * when fstat is used on large files.  This function is
+ * an attempt to get around that.
+ *
+ * @par fd File Descriptor.
+ * @return -1 on error, length of file (in bytes) otherwise.
+ */
+static size_t nc_get_filelen(const int fd) {
+
+  size_t flen;
+
+#ifdef HAVE_FILE_LENGTH_I64
+  __int64 file_len = 0;
+  if ((file_len = _filelengthi64(fd)) < 0) {
+    return file_len;
+  }
+  flen = (size_t)file_len;
+
+#else
+  int res = 0;
+  struct stat sb;
+  if((res = fstat(fd,&sb)) <0)
+    return res;
+
+  flen = sb.st_size;
+#endif
+
+  return flen;
+
+}
+
+
 /*
  * What is the system pagesize?
  */
@@ -159,7 +195,7 @@ fgrow(const int fd, const off_t len)
 	if (fstat(fd, &sb) < 0)
 		return errno;
 	if (len < sb.st_size)
-		return ENOERR;
+		return NC_NOERR;
 	{
 	    const long dumb = 0;
 	    /* we don't use ftruncate() due to problem with FAT32 file systems */
@@ -174,7 +210,7 @@ fgrow(const int fd, const off_t len)
 	    if (lseek(fd, pos, SEEK_SET) < 0)
 		return errno;
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -186,13 +222,23 @@ fgrow(const int fd, const off_t len)
 static int
 fgrow2(const int fd, const off_t len)
 {
-	struct stat sb;
-	if (fstat(fd, &sb) < 0)
-		return errno;
-	if (len <= sb.st_size)
-		return ENOERR;
-	{
-	    const char dumb = 0;
+
+
+  /* There is a problem with fstat on Windows based systems
+     which manifests (so far) when Config RELEASE is built.
+     Use _filelengthi64 isntead.
+
+     See https://github.com/Unidata/netcdf-c/issues/188
+
+  */
+
+
+  size_t file_len = nc_get_filelen(fd);
+  if(file_len < 0) return errno;
+  if(len <= file_len)
+    return NC_NOERR;
+  {
+    const char dumb = 0;
 	    /* we don't use ftruncate() due to problem with FAT32 file systems */
 	    /* cache current position */
 	    const off_t pos = lseek(fd, 0, SEEK_CUR);
@@ -205,7 +251,7 @@ fgrow2(const int fd, const off_t len)
 	    if (lseek(fd, pos, SEEK_SET) < 0)
 		return errno;
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 /* End OS */
 /* Begin px */
@@ -261,7 +307,7 @@ px_pgout(ncio *const nciop,
 	    return errno;
 	*posp += extent;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /*! Read in a page of data.
@@ -282,7 +328,9 @@ px_pgin(ncio *const nciop,
 {
 	int status;
 	ssize_t nread;
-
+    size_t read_count = 0;
+    ssize_t bytes_xfered = 0;
+    void *p = vp;
 #ifdef X_ALIGN
 	assert(offset % X_ALIGN == 0);
 	assert(extent % X_ALIGN == 0);
@@ -309,19 +357,37 @@ px_pgin(ncio *const nciop,
 	}
 
 	errno = 0;
-	nread = read(nciop->fd, vp, extent);
-	if(nread != (ssize_t) extent)
-	{
-		status = errno;
-		if(nread == -1 || status != ENOERR)
-			return status;
-		/* else it's okay we read less than asked for */
-		(void) memset((char *)vp + nread, 0, (ssize_t)extent - nread);
-	}
-	*nreadp = nread;
+    /* Handle the case where the read is interrupted
+       by a signal (see NCF-337,
+       http://pubs.opengroup.org/onlinepubs/009695399/functions/read.html)
+
+       When this happens, nread will (should) be the bytes read, and
+       errno will be set to EINTR.  On older systems nread might be -1.
+       If this is the case, there's not a whole lot we can do about it
+       as we can't compute any offsets, so we will attempt to read again.
+       This *feels* like it could lead to an infinite loop, but it shouldn't
+       unless the read is being constantly interrupted by a signal, and is
+       on an older system which returns -1 instead of bytexs read.
+
+       The case where it's a short read is already handled by the function
+       (according to the comment below, at least). */
+    do {
+      nread = read(nciop->fd,vp,extent);
+    } while (nread == -1 && errno == EINTR);
+
+
+    if(nread != (ssize_t)extent) {
+      status = errno;
+      if( nread == -1 || (status != EINTR && status != NC_NOERR))
+        return status;
+      /* else it's okay we read less than asked for */
+      (void) memset((char *)vp + nread, 0, (ssize_t)extent - nread);
+    }
+
+    *nreadp = nread;
 	*posp += nread;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* This struct is for POSIX systems, with NC_SHARE not in effect. If
@@ -367,7 +433,7 @@ typedef struct ncio_px {
    offset - file offset for beginning of to region to be
    released.
 
-   rflags - only RGN_MODIFIED is relevent to this function, others ignored
+   rflags - only RGN_MODIFIED is relevant to this function, others ignored
 */
 static int
 px_rel(ncio_px *const pxp, off_t offset, int rflags)
@@ -383,7 +449,7 @@ px_rel(ncio_px *const pxp, off_t offset, int rflags)
 	}
 	pxp->bf_refcount--;
 
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /* This function indicates the file region starting at offset may be
@@ -400,7 +466,7 @@ px_rel(ncio_px *const pxp, off_t offset, int rflags)
    nciop - pointer to ncio struct.
    offset - num bytes from beginning of buffer to region to be
    released.
-   rflags - only RGN_MODIFIED is relevent to this function, others ignored
+   rflags - only RGN_MODIFIED is relevant to this function, others ignored
 */
 static int
 ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
@@ -424,7 +490,7 @@ ncio_px_rel(ncio *const nciop, off_t offset, int rflags)
    offset - start byte of region to get.
    extent - how many bytes to read.
    rflags - One of the RGN_* flags defined in ncio.h.
-   vpp - pointer to pointer that will recieve data.
+   vpp - pointer to pointer that will receive data.
 
    NOTES:
 
@@ -454,7 +520,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 		int rflags,
 		void **const vpp)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	const off_t blkoffset = _RNDDOWN(offset, (off_t)pxp->blksz);
 	off_t diff = (size_t)(offset - blkoffset);
@@ -497,7 +563,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 				 middle,
 				 &pxp->bf_cnt,
 				 &pxp->pos);
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			pxp->bf_extent = 2 * pxp->blksz;
 			pxp->bf_cnt += pxp->blksz;
@@ -532,7 +598,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->blksz,
 					pxp->bf_base,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 			pxp->bf_cnt -= pxp->blksz;
@@ -551,7 +617,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->blksz,
 					pxp->bf_base,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 		}
@@ -569,7 +635,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 				 middle,
 				 &pxp->bf_cnt,
 				 &pxp->pos);
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			pxp->bf_extent = 2 * pxp->blksz;
 			pxp->bf_cnt += pxp->blksz;
@@ -597,7 +663,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 					pxp->bf_cnt - pxp->blksz,
 					middle,
 					&pxp->pos);
-				if(status != ENOERR)
+				if(status != NC_NOERR)
 					return status;
 			}
 			pxp->bf_cnt = pxp->blksz;
@@ -616,7 +682,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 			 pxp->bf_base,
 			 &pxp->bf_cnt,
 			 &pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_offset = blkoffset;
 		if(upper_cnt != 0)
@@ -641,7 +707,7 @@ px_get(ncio *const nciop, ncio_px *const pxp,
 			pxp->bf_cnt,
 			pxp->bf_base,
 			&pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_rflags = 0;
 	}
@@ -653,7 +719,7 @@ pgin:
 		 pxp->bf_base,
 		 &pxp->bf_cnt,
 		 &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 	 pxp->bf_offset = blkoffset;
 	 pxp->bf_extent = blkextent;
@@ -667,8 +733,12 @@ done:
 	pxp->bf_rflags |= rflags;
 	pxp->bf_refcount++;
 
-	*vpp = (char *)pxp->bf_base + diff;
-	return ENOERR;
+#ifndef __CHAR_UNSIGNED__
+    *vpp = (void *)((char *)pxp->bf_base + diff);
+#else
+    *vpp = (void *)((signed char*)pxp->bf_base + diff);
+#endif
+	return NC_NOERR;
 }
 
 /* Request that the region (offset, extent) be made available through
@@ -725,7 +795,7 @@ px_double_buffer(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	void *src;
 	void *dest;
 
@@ -735,7 +805,7 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 #endif
 	status = px_get(nciop, pxp, to, nbytes, RGN_WRITE,
 			&dest);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(pxp->slave == NULL)
@@ -762,7 +832,7 @@ fprintf(stderr, "\tdouble_buffr %ld %ld %ld\n",
 	pxp->slave->pos = pxp->pos;
 	status = px_get(nciop, pxp->slave, from, nbytes, 0,
 			&src);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 	if(pxp->pos != pxp->slave->pos)
 	{
@@ -797,7 +867,7 @@ ncio_px_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	off_t lower;
 	off_t upper;
 	char *base;
@@ -805,7 +875,7 @@ ncio_px_move(ncio *const nciop, off_t to, off_t from,
 	size_t extent;
 
 	if(to == from)
-		return ENOERR; /* NOOP */
+		return NC_NOERR; /* NOOP */
 
 	if(fIsSet(rflags, RGN_WRITE) && !fIsSet(nciop->ioflags, NC_WRITE))
 		return EPERM; /* attempt to write readonly file */
@@ -847,7 +917,7 @@ if(to > from)
 
 			status = px_double_buffer(nciop, toh, frm,
 				 	loopextent, rflags) ;
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			remaining -= loopextent;
 
@@ -863,7 +933,7 @@ else
 
 			status = px_double_buffer(nciop, to, from,
 				 	loopextent, rflags) ;
-			if(status != ENOERR)
+			if(status != NC_NOERR)
 				return status;
 			remaining -= loopextent;
 
@@ -873,7 +943,7 @@ else
 			from += loopextent;
 		}
 }
-		return ENOERR;
+		return NC_NOERR;
 	}
 
 #if INSTRUMENT
@@ -882,7 +952,7 @@ fprintf(stderr, "\tncio_px_move small\n");
 	status = px_get(nciop, pxp, lower, extent, RGN_WRITE|rflags,
 			(void **)&base);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(to > from)
@@ -903,14 +973,14 @@ static int
 ncio_px_sync(ncio *const nciop)
 {
 	ncio_px *const pxp = (ncio_px *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(fIsSet(pxp->bf_rflags, RGN_MODIFIED))
 	{
 		assert(pxp->bf_refcount <= 0);
 		status = px_pgout(nciop, pxp->bf_offset,
 			pxp->bf_cnt,
 			pxp->bf_base, &pxp->pos);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			return status;
 		pxp->bf_rflags = 0;
 	}
@@ -1004,7 +1074,7 @@ ncio_px_init2(ncio *const nciop, size_t *sizehintp, int isNew)
 		pxp->bf_extent = bufsz;
 		(void) memset(pxp->bf_base, 0, pxp->bf_extent);
 	}
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1074,7 +1144,7 @@ static int
 ncio_spx_rel(ncio *const nciop, off_t offset, int rflags)
 {
 	ncio_spx *const pxp = (ncio_spx *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	assert(pxp->bf_offset <= offset);
 	assert(pxp->bf_cnt != 0);
@@ -1123,7 +1193,7 @@ ncio_spx_get(ncio *const nciop,
 		void **const vpp)
 {
 	ncio_spx *const pxp = (ncio_spx *)nciop->pvt;
-	int status = ENOERR;
+	int status = NC_NOERR;
 #ifdef X_ALIGN
 	size_t rem;
 #endif
@@ -1145,9 +1215,9 @@ ncio_spx_get(ncio *const nciop,
 	}
 
 	{
-		const size_t rndup = extent % X_ALIGN;
-		if(rndup != 0)
-			extent += X_ALIGN - rndup;
+      const size_t rndup = extent % X_ALIGN;
+      if(rndup != 0)
+        extent += X_ALIGN - rndup;
 	}
 
 	assert(offset % X_ALIGN == 0);
@@ -1163,7 +1233,7 @@ ncio_spx_get(ncio *const nciop,
 			pxp->bf_extent = 0;
 		}
 		assert(pxp->bf_extent == 0);
-		pxp->bf_base = malloc(extent);
+		pxp->bf_base = malloc(extent+1);
 		if(pxp->bf_base == NULL)
 			return ENOMEM;
 		pxp->bf_extent = extent;
@@ -1173,7 +1243,7 @@ ncio_spx_get(ncio *const nciop,
 		 extent,
 		 pxp->bf_base,
 		 &pxp->bf_cnt, &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	pxp->bf_offset = offset;
@@ -1186,7 +1256,7 @@ ncio_spx_get(ncio *const nciop,
 #else
 	*vpp = pxp->bf_base;
 #endif
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1197,7 +1267,7 @@ strategy(ncio *const nciop, off_t to, off_t offset,
 			size_t extent, int rflags)
 {
 	static ncio_spx pxp[1];
-	int status = ENOERR;
+	int status = NC_NOERR;
 #ifdef X_ALIGN
 	size_t rem;
 #endif
@@ -1247,7 +1317,7 @@ fprintf(stderr, "strategy %ld at %ld to %ld\n",
 		 extent,
 		 pxp->bf_base,
 		 &pxp->bf_cnt, &pxp->pos);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	pxp->bf_offset = to; /* TODO: XALIGN */
@@ -1280,7 +1350,7 @@ static int
 ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 			size_t nbytes, int rflags)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	off_t lower = from;
 	off_t upper = to;
 	char *base;
@@ -1290,7 +1360,7 @@ ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 	rflags &= RGN_NOLOCK; /* filter unwanted flags */
 
 	if(to == from)
-		return ENOERR; /* NOOP */
+		return NC_NOERR; /* NOOP */
 
 	if(to > from)
 	{
@@ -1311,7 +1381,7 @@ ncio_spx_move(ncio *const nciop, off_t to, off_t from,
 	status = ncio_spx_get(nciop, lower, extent, RGN_WRITE|rflags,
 			(void **)&base);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		return status;
 
 	if(to > from)
@@ -1332,7 +1402,7 @@ static int
 ncio_spx_sync(ncio *const nciop)
 {
 	/* NOOP */
-	return ENOERR;
+	return NC_NOERR;
 }
 
 static void
@@ -1380,7 +1450,7 @@ ncio_spx_init2(ncio *const nciop, const size_t *const sizehintp)
 		return ENOMEM;
 	}
 	/* else */
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1497,7 +1567,7 @@ ncio_px_new(const char *path, int ioflags)
 #define NC_DEFAULT_CREAT_MODE 0666
 #endif
 
-/* Create a file, and the ncio struct to go with it. This funtion is
+/* Create a file, and the ncio struct to go with it. This function is
    only called from nc__create_mp.
 
    path - path of file to create.
@@ -1516,6 +1586,7 @@ int
 posixio_create(const char *path, int ioflags,
 	size_t initialsz,
 	off_t igeto, size_t igetsz, size_t *sizehintp,
+	void* parameters,
 	ncio **nciopp, void **const igetvpp)
 {
 	ncio *nciop;
@@ -1579,13 +1650,13 @@ posixio_create(const char *path, int ioflags,
 	else
 		status = ncio_px_init2(nciop, sizehintp, 1);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(initialsz != 0)
 	{
 		status = fgrow(fd, (off_t)initialsz);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
@@ -1595,12 +1666,12 @@ posixio_create(const char *path, int ioflags,
 				igeto, igetsz,
                         	RGN_WRITE,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) close(fd);
@@ -1659,6 +1730,7 @@ int
 posixio_open(const char *path,
 	int ioflags,
 	off_t igeto, size_t igetsz, size_t *sizehintp,
+        void* parameters,
 	ncio **nciopp, void **const igetvpp)
 {
 	ncio *nciop;
@@ -1674,7 +1746,7 @@ posixio_open(const char *path,
 		return ENOMEM;
 
 #ifdef O_BINARY
-	//#if _MSC_VER
+	/*#if _MSC_VER*/
 	fSet(oflags, O_BINARY);
 #endif
 
@@ -1710,7 +1782,7 @@ posixio_open(const char *path,
 	else
 		status = ncio_px_init2(nciop, sizehintp, 0);
 
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 		goto unwind_open;
 
 	if(igetsz != 0)
@@ -1719,12 +1791,12 @@ posixio_open(const char *path,
 				igeto, igetsz,
                         	0,
                         	igetvpp);
-		if(status != ENOERR)
+		if(status != NC_NOERR)
 			goto unwind_open;
 	}
 
 	*nciopp = nciop;
-	return ENOERR;
+	return NC_NOERR;
 
 unwind_open:
 	(void) close(fd); /* assert fd >= 0 */
@@ -1761,7 +1833,7 @@ ncio_px_filesize(ncio *nciop, off_t *filesizep)
 	return errno;
     *filesizep = sb.st_size;
 #endif
-	return ENOERR;
+	return NC_NOERR;
 }
 
 /*
@@ -1775,7 +1847,7 @@ static int
 ncio_px_pad_length(ncio *nciop, off_t length)
 {
 
-	int status = ENOERR;
+	int status = NC_NOERR;
 
 	if(nciop == NULL)
 		return EINVAL;
@@ -1784,13 +1856,13 @@ ncio_px_pad_length(ncio *nciop, off_t length)
 	        return EPERM; /* attempt to write readonly file */
 
 	status = nciop->sync(nciop);
-	if(status != ENOERR)
+	if(status != NC_NOERR)
 	        return status;
 
  	status = fgrow2(nciop->fd, length);
- 	if(status != ENOERR)
+ 	if(status != NC_NOERR)
 	        return status;
-	return ENOERR;
+	return NC_NOERR;
 }
 
 
@@ -1807,7 +1879,7 @@ ncio_px_pad_length(ncio *nciop, off_t length)
 static int
 ncio_px_close(ncio *nciop, int doUnlink)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(nciop == NULL)
 		return EINVAL;
 	if(nciop->fd > 0) {
@@ -1823,7 +1895,7 @@ ncio_px_close(ncio *nciop, int doUnlink)
 static int
 ncio_spx_close(ncio *nciop, int doUnlink)
 {
-	int status = ENOERR;
+	int status = NC_NOERR;
 	if(nciop == NULL)
 		return EINVAL;
 	if(nciop->fd > 0) {
