@@ -19,11 +19,20 @@ conditions.
 #include "H5DSpublic.h"
 #include <utf8proc.h>
 
-#if 0 /*def USE_PNETCDF*/
-#include <pnetcdf.h>
+#define MEGABYTE 1048576
+
+#undef DEBUGH5
+
+#ifdef DEBUGH5
+/* Provide a catchable error reporting function */
+static herr_t
+h5catch(void* ignored)
+{
+    H5Eprint(NULL);
+    return 0;
+}
 #endif
 
-#define MEGABYTE 1048576
 
 /* These are the default chunk cache sizes for HDF5 files created or
  * opened with netCDF-4. */
@@ -45,6 +54,33 @@ int nc_log_level = -1;
 
 #endif /* LOGGING */
 
+int nc4_hdf5_initialized = 0;
+
+/* Provide a wrapper for H5Eset_auto */
+static herr_t
+set_auto(void* func, void *client_data) 
+{
+#ifdef DEBUGH5
+    return H5Eset_auto2(H5E_DEFAULT,(H5E_auto2_t)h5catch,client_data);
+#else
+    return H5Eset_auto2(H5E_DEFAULT,(H5E_auto2_t)func,client_data);
+#endif
+}
+
+/*
+Provide a function to do any necessary initialization
+of the HDF5 library.
+*/
+
+void
+nc4_hdf5_initialize(void)
+{
+    if (set_auto(NULL, NULL) < 0)
+	LOG((0, "Couldn't turn off HDF5 error messages!"));
+    LOG((1, "HDF5 error messages have been turned off."));
+    nc4_hdf5_initialized = 1;
+}
+
 /* Check and normalize and name. */
 int
 nc4_check_name(const char *name, char *norm_name)
@@ -238,16 +274,6 @@ nc4_find_nc_grp_h5(int ncid, NC **nc, NC_GRP_INFO_T **grpp,
     if(f == NULL) return NC_EBADID;
     *nc = f;
 
-#if 0 /* I do not understand this code at all */
-#if 0 /*def USE_PNETCDF*/
-    if (h5->pnetcdf_file) {
-      *h5p = NULL;
-      *grpp = NULL;
-      return NC_NOERR;
-    }
-#endif
-#endif
-
     if (h5) {
 	assert(h5->root_grp);
 	/* If we can't find it, the grp id part of ncid is bad. */
@@ -529,10 +555,13 @@ nc4_find_grp_att(NC_GRP_INFO_T *grp, int varid, const char *name, int attnum,
 
    /* Now find the attribute by name or number. If a name is provided,
     * ignore the attnum. */
-   for (*att = attlist; *att; *att = (*att)->l.next)
-      if ((name && !strcmp((*att)->name, name)) ||
-	  (!name && (*att)->attnum == attnum))
-	 return NC_NOERR;
+   if(attlist)
+       for (*att = attlist; *att; *att = (*att)->l.next) {
+           if (name && (*att)->name && !strcmp((*att)->name, name))
+	       return NC_NOERR;
+           if (!name && (*att)->attnum == attnum)
+	       return NC_NOERR;
+       }
 
    /* If we get here, we couldn't find the attribute. */
    return NC_ENOTATT;
@@ -747,7 +776,8 @@ nc4_check_dup_name(NC_GRP_INFO_T *grp, char *name)
    NC_TYPE_INFO_T *type;
    NC_GRP_INFO_T *g;
    NC_VAR_INFO_T *var;
-
+   uint32_t hash;
+   
    /* Any types of this name? */
    for (type = grp->type; type; type = type->l.next)
       if (!strcmp(type->name, name))
@@ -759,8 +789,9 @@ nc4_check_dup_name(NC_GRP_INFO_T *grp, char *name)
 	 return NC_ENAMEINUSE;
 
    /* Any variables of this name? */
+   hash =  hash_fast(name, strlen(name));
    for (var = grp->var; var; var = var->l.next)
-      if (!strcmp(var->name, name))
+      if (var->hash == hash && !strcmp(var->name, name))
 	 return NC_ENAMEINUSE;
 
    return NC_NOERR;
@@ -1387,12 +1418,15 @@ nc4_normalize_name(const char *name, char *norm_name)
 int
 nc_set_log_level(int new_level)
 {
+   if(!nc4_hdf5_initialized)
+	nc4_hdf5_initialize();
+
    /* If the user wants to completely turn off logging, turn off HDF5
       logging too. Now I truely can't think of what to do if this
       fails, so just ignore the return code. */
    if (new_level == NC_TURN_OFF_LOGGING)
    {
-      H5Eset_auto(NULL, NULL);
+      set_auto(NULL,NULL);
       LOG((1, "HDF5 error messages turned off!"));
    }
 
@@ -1400,7 +1434,7 @@ nc_set_log_level(int new_level)
    if (new_level > NC_TURN_OFF_LOGGING &&
        nc_log_level <= NC_TURN_OFF_LOGGING)
    {
-      if (H5Eset_auto((H5E_auto_t)&H5Eprint, stderr) < 0)
+      if (set_auto((H5E_auto_t)&H5Eprint, stderr) < 0)
 	 LOG((0, "H5Eset_auto failed!"));
       LOG((1, "HDF5 error messages turned on."));
    }
@@ -1454,9 +1488,9 @@ rec_print_metadata(NC_GRP_INFO_T *grp, int tab_count)
              strcat(dims_string, temp_string);
            }
       }
-      LOG((2, "%s VARIABLE - varid: %d name: %s type: %d ndims: %d dimscale: %d dimids:%s",
+      LOG((2, "%s VARIABLE - varid: %d name: %s type: %d ndims: %d dimscale: %d dimids:%s endianness: %d, hdf_typeid: %d",
 	   tabs, var->varid, var->name, var->type_info->nc_typeid, var->ndims, (int)var->dimscale,
-	   (dims_string ? dims_string : " -")));
+       (dims_string ? dims_string : " -"),var->type_info->endianness, var->type_info->native_hdf_typeid));
       for(att = var->att; att; att = att->l.next)
 	 LOG((2, "%s VAR ATTRIBUTE - attnum: %d name: %s type: %d len: %d",
 	      tabs, att->attnum, att->name, att->nc_typeid, att->len));
