@@ -583,7 +583,7 @@ than once, then the last occurrence is used; this is so
 that is possible to forcibly override user specified
 parameters by suffixing.  IMPORTANT: client parameter string
 is assumed to have blanks compressed out.  Returns 1 if parse
-suceeded, 0 otherwise; */
+succeeded, 0 otherwise; */
 
 int
 ocuridecodeparams(OCURI* ocuri)
@@ -593,12 +593,10 @@ ocuridecodeparams(OCURI* ocuri)
     int nparams;
     char* params = NULL;
     char** plist;
-    size_t len;
 
     if(ocuri == NULL) return 0;
     if(ocuri->params == NULL) return 1;
 
-    len = strlen(ocuri->params);
     params = strdup(ocuri->params);
     if(params == NULL)
 	return 0; /* no memory */
